# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
""" AWS auth functions from https://github.com/salesforce/cloudsplaining/blob/master/cloudsplaining/shared/aws_login.py """
import os
import logging
import boto3
import sys
from botocore.config import Config
from metabadger.shared import utils

logger = logging.getLogger(__name__)


def get_boto3_client(
    service: str, profile: str = None, region: str = "us-west-2"
) -> boto3.Session.client:
    """Get a boto3 client for a given service"""
    logging.getLogger("botocore").setLevel(logging.CRITICAL)
    session_data = {"region_name": region}
    if profile:
        session_data["profile_name"] = profile
    session = boto3.Session(**session_data)
    if region not in get_available_regions(service):
        utils.print_red(f"The service {service} is not available in this region!")
        sys.exit()
    config = Config(read_timeout=5, connect_timeout=5, retries={"max_attempts": 10})
    if os.environ.get("LOCALSTACK_ENDPOINT_URL"):
        client = session.client(
            service,
            config=config,
            endpoint_url=os.environ.get("LOCALSTACK_ENDPOINT_URL"),
        )
    else:
        client = session.client(service, config=config)
    # Uncomment if you are having trouble with authing across a particular region :)
    # utils.print_green(f"Authentication to {service} in {region} was successful!")
    logger.debug(
        f"{client.meta.endpoint_url} in {client.meta.region_name}: boto3 client login successful"
    )
    return client


def get_boto3_resource(
    service: str, profile: str = None, region: str = "us-west-2"
) -> boto3.Session.resource:
    """Get a boto3 resource for a given service"""
    logging.getLogger("botocore").setLevel(logging.CRITICAL)
    session_data = {"region_name": region}
    if profile:
        session_data["profile_name"] = profile
    session = boto3.Session(**session_data)

    resource = session.resource(service)
    return resource


def get_current_account_id(sts_client: boto3.Session.client) -> str:
    """Get the current account ID"""
    response = sts_client.get_caller_identity()
    current_account_id = response.get("Account")
    return current_account_id


def get_available_regions(service: str):
    """AWS exposes their list of regions as an API. Gather the list."""
    regions = boto3.session.Session().get_available_regions(service)
    logger.debug(
        "The service %s does not have available regions. Returning us-west-2 as default"
    )
    if not regions:
        regions = ["us-west-2"]
    return regions



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
import click


def click_validate_tag_alphanumeric(ctx, param, value):
    if value is not None:
        try:
            if value == "":
                return []
            else:
                tag_values_to_check = value.split(",")
                return tag_values_to_check
        except ValueError:
            raise click.BadParameter(
                "Supply the list of tag names to include for hardening in a comma separated string."
            )



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
import boto3
import click
import time
from collections import Counter
from typing import Tuple
from metabadger.shared import utils


def get_summary(ec2_client: boto3.Session.client):
    total_instances = 0
    instance_tracker = []
    paginator = ec2_client.get_paginator("describe_instances")
    instances = paginator.paginate(
        PaginationConfig={"PageSize": 1000}
    ).build_full_result()
    with click.progressbar(
        instances["Reservations"],
        label=utils.convert_green("Calculating instance metadata summary..."),
    ) as all_instances:
        for each_reservation in all_instances:
            time.sleep(0.001)
            for each_instance in each_reservation["Instances"]:
                instance_tracker.append((each_instance["MetadataOptions"]["State"]))
                instance_tracker.append(
                    (each_instance["MetadataOptions"]["HttpTokens"])
                )
                instance_tracker.append(
                    (each_instance["MetadataOptions"]["HttpEndpoint"])
                )
                total_instances += 1
    instance_options = Counter(instance_tracker)
    instance_options["instances"] = total_instances
    return instance_options


def discover_instances(ec2_client: boto3.Session.client) -> list:
    """Get a list of instances, both running and stopped"""
    paginator = ec2_client.get_paginator("describe_instances")
    instances = paginator.paginate(
        PaginationConfig={"PageSize": 1000}
    ).build_full_result()
    instance_list = []
    with click.progressbar(
        instances["Reservations"],
        label=utils.convert_green("Pulling instances...                    "),
    ) as all_instances:
        for each_reservation in all_instances:
            time.sleep(0.001)
            for each_instance in each_reservation["Instances"]:
                instance_list.append(each_instance["InstanceId"])
    return instance_list


def discover_roles(ec2_client: boto3.Session.client) -> Tuple[dict, dict]:
    """Get a summary of roles attached to instances"""
    instances = ec2_client.describe_instances()
    role_count = 0
    instance_count = 0
    instance_role_summary = {}
    for ins_id in instances["Reservations"]:
        instance_id = ins_id["Instances"][0]["InstanceId"]
        instance_count += 1
        instance_name = None
        try:
            for tag in ins_id["Instances"][0]["Tags"]:
                if tag["Key"] == "Name":
                    instance_name = tag["Value"]
        except:
            instance_name = "N/A"
        try:
            role_name = ins_id["Instances"][0]["IamInstanceProfile"]["Arn"]
            role_count += 1
        except:
            role_name = "N/A"
        if role_name != "N/A":
            instance_role_summary[instance_id] = {
                "instance_name": instance_name,
                "instance_role": role_name,
            }
    short_summary = {"role_count": role_count, "instance_count": instance_count}
    return instance_role_summary, short_summary


def get_instance_tags(ec2_client: boto3.Session.client, instance_id: str):
    """Get instance tags to parse through for selective hardening"""
    tag_values = []
    tags = ec2_client.describe_tags(
        Filters=[
            {
                "Name": "resource-id",
                "Values": [
                    instance_id,
                ],
            },
        ],
    )["Tags"]
    for tag in tags:
        tag_values.append(tag["Value"])
    return tag_values



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
"""Harden the AWS metadata service to your liking by upgrading to IMDSv2"""
import click
from tabulate import tabulate
from metabadger.shared import utils, aws_auth, validate, discover, modify
from metabadger.command import discover_role_usage


@click.command(short_help="Harden the AWS instance metadata service from v1 to v2")
@click.option(
    "--all-region",
    "-a",
    is_flag=True,
    default=False,
    help="Update IMDS on all available regions in the AWS account",
)
@click.option(
    "--exclusion",
    "-e",
    is_flag=True,
    default=False,
    help="The exclusion flag will apply to everything besides what is specified, tags or instances",
)
@click.option(
    "--dry-run",
    "-d",
    is_flag=True,
    default=False,
    help="Dry run of hardening metadata changes",
)
@click.option(
    "--v1",
    "-v1",
    is_flag=True,
    default=False,
    help="Enforces v1 of the metadata service",
)
@click.option(
    "--input-file",
    "-i",
    type=click.Path(exists=True),
    required=False,
    help="Path of csv file of instances to harden IMDS for",
)
@click.option(
    "--tags",
    "-t",
    "tags",
    type=str,
    default="",
    help="A comma separated list of tags to apply the hardening setting to",
    callback=validate.click_validate_tag_alphanumeric,
)
@click.option(
    "--region",
    "-r",
    "region",
    type=str,
    required=False,
    default="us-west-2",
    help="Specify which AWS region you will perform this command in",
)
@click.option(
    "--profile", "-p", type=str, required=False, help="Specify the AWS IAM profile."
)
def harden_metadata(
    dry_run: bool,
    v1: bool,
    input_file: str,
    tags: str,
    profile: str,
    region: str,
    exclusion: bool,
    all_region: bool,
):
    ec2_resource = aws_auth.get_boto3_resource(
        region=region, profile=profile, service="ec2"
    )
    ec2_client = aws_auth.get_boto3_client(
        region=region, profile=profile, service="ec2"
    )
    all_regions = aws_auth.get_available_regions("ec2")
    instance_list = discover.discover_instances(ec2_client)
    if discover.discover_roles(ec2_client)[1]["role_count"] > 0:
        click.confirm(
            utils.convert_red(
                f"Warning: One or more of the instances in {ec2_client.meta.region_name} you want to upgrade has a role attached, do you want to continue?"
            ),
            abort=True,
        )
    if discover.discover_roles(ec2_client)[1]["instance_count"] <= 0:
        utils.print_yellow(f"No EC2 instances found in region: {region}")
    if dry_run:
        utils.print_yellow(
            "Running in dry run mode, this will NOT make any changes to your metadata service"
        )
    if v1:
        message_default = "V1 Enforced"
        http_option = "optional"
    else:
        message_default = "V2 Enforced"
        http_option = "required"
    if exclusion and input_file:
        utils.print_yellow("Excluding instances specified in your configuration file")
        data = utils.read_from_csv(input_file)
        print(f"Reading instances from input csv file\n{data}")
        delta = [instance for instance in instance_list if instance not in data]
        for instance in delta:
            modify.metamodify(
                ec2_client,
                message_default,
                http_option,
                "enabled",
                instance,
                dry_run,
                region,
            )
    elif all_region and tags and exclusion:
        tag_apply_count = 0
        utils.print_yellow("All region IMDS hardening initiated")
        for region_iterate in all_regions:
            try:
                region_client = aws_auth.get_boto3_client(
                    region=region_iterate, profile=profile, service="ec2"
                )
                region_instance_list = discover.discover_instances(region_client)
                for instance in region_instance_list:
                    if not any(
                        value in discover.get_instance_tags(region_client, instance)
                        for value in tags
                    ):
                        modify.metamodify(
                            region_client,
                            message_default,
                            http_option,
                            "enabled",
                            instance,
                            dry_run,
                            region_iterate,
                        )
            except Exception as e:
                utils.print_yellow(f"No instance information for {region_iterate}")
            if tag_apply_count < 1:
                print(f"No instances with this tag found, no changes were made")
    elif exclusion and tags:
        utils.print_yellow("Excluding instances specified by tags")
        print(f"Tags: {tags}")
        for instance in instance_list:
            if not any(
                value in discover.get_instance_tags(ec2_client, instance)
                for value in tags
            ):
                modify.metamodify(
                    ec2_client,
                    message_default,
                    http_option,
                    "enabled",
                    instance,
                    dry_run,
                    region,
                )
    elif exclusion:
        utils.print_yellow("An exclusion requires either tags or instance list")
        raise click.Abort()
    elif input_file:
        data = utils.read_from_csv(input_file)
        print(f"Reading instances from input csv file\n{data}")
        for instance in data:
            modify.metamodify(
                ec2_client,
                message_default,
                http_option,
                "enabled",
                instance,
                dry_run,
                region,
            )
    elif all_region and tags:
        tag_apply_count = 0
        utils.print_yellow("All region IMDS hardening initiated")
        for region_iterate in all_regions:
            try:
                region_client = aws_auth.get_boto3_client(
                    region=region_iterate, profile=profile, service="ec2"
                )
                region_instance_list = discover.discover_instances(region_client)
                for instance in region_instance_list:
                    if any(
                        value in discover.get_instance_tags(region_client, instance)
                        for value in tags
                    ):
                        modify.metamodify(
                            region_client,
                            message_default,
                            http_option,
                            "enabled",
                            instance,
                            dry_run,
                            region_iterate,
                        )
            except Exception as e:
                utils.print_yellow(f"No instance information for {region_iterate}")
            if tag_apply_count < 1:
                print(f"No instances with this tag found, no changes were made")
    elif all_region:
        utils.print_yellow("All region IMDS hardening initiated")
        for region_iterate in all_regions:
            try:
                region_client = aws_auth.get_boto3_client(
                    region=region_iterate, profile=profile, service="ec2"
                )
                region_instance_list = discover.discover_instances(region_client)
                for instance in region_instance_list:
                    modify.metamodify(
                        region_client,
                        message_default,
                        http_option,
                        "enabled",
                        instance,
                        dry_run,
                        region_iterate,
                    )
            except Exception as e:
                utils.print_yellow(f"No instance information for {region_iterate}")
    elif tags:
        tag_apply_count = 0
        utils.print_yellow("Only applying hardening to the following tagged instances")
        print(f"Tags: {tags}")
        for instance in instance_list:
            if any(
                value in discover.get_instance_tags(ec2_client, instance)
                for value in tags
            ):
                tag_apply_count += 1
                modify.metamodify(
                    ec2_client,
                    message_default,
                    http_option,
                    "enabled",
                    instance,
                    dry_run,
                    region,
                )
        if tag_apply_count < 1:
            print(f"No instances with this tag found, no changes were made")
    else:
        for instance in instance_list:
            modify.metamodify(
                ec2_client,
                message_default,
                http_option,
                "enabled",
                instance,
                dry_run,
                region,
            )



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
"""Discover EC2 Instance Metadata usage in your AWS account"""
import click
import time
from collections import Counter
from tabulate import tabulate
from metabadger.shared import utils, aws_auth, discover


@click.command(short_help="Discover summary of IMDS service usage within EC2")
@click.option(
    "--all-region",
    "-a",
    is_flag=True,
    default=False,
    help="Provide a metadata summary for all available regions in the AWS account",
)
@click.option(
    "--json",
    "-j",
    is_flag=True,
    default=False,
    help="Get metadata summary in JSON format",
)
@click.option(
    "--region",
    "-r",
    "region",
    type=str,
    required=False,
    default="us-west-2",
    help="Specify which AWS region you will perform this command in",
)
@click.option(
    "--profile", "-p", type=str, required=False, help="Specify the AWS IAM profile."
)
def discover_metadata(json, profile: str, region: str, all_region):
    ec2_resource = aws_auth.get_boto3_resource(
        region=region, profile=profile, service="ec2"
    )
    ec2_client = aws_auth.get_boto3_client(
        region=region, profile=profile, service="ec2"
    )
    all_regions = aws_auth.get_available_regions("ec2")
    if all_region:
        instance_total_options = []
        utils.print_green(
            f"All region scan initiated: Iterating through each of the following regions {utils.convert_yellow(all_regions)}"
        )
        for each_region in all_regions:
            try:
                instance_total_options.append(
                    discover.get_summary(
                        aws_auth.get_boto3_client(
                            region=each_region, profile=profile, service="ec2"
                        )
                    )
                )
            except Exception as e:
                utils.print_yellow(f"No instance information for {each_region}")
        instance_all_region_total = sum(instance_total_options, Counter())
        if instance_all_region_total["instances"] == 0:
            utils.print_red(f"No instances found in your AWS account")
        else:
            enforcement = (
                float(
                    instance_all_region_total["required"]
                    / instance_all_region_total["instances"]
                )
                * 100
            )
            if not json:
                instance_all_region_total[
                    "percent_enforcement_v2"
                ] = utils.convert_green(f"{enforcement:.2f}%")
                return utils.pretty_metadata_summary([dict(instance_all_region_total)])
            elif json:
                instance_all_region_total["percent_enforcement_v2"] = enforcement
                print(utils.pretty_json_summary(instance_all_region_total))
    else:
        instance_list = discover.discover_instances(ec2_client)
        if not instance_list:
            utils.print_yellow(f"No instances found in region: {region}")
        else:
            print(f"Gathering EC2 metrics for {region}...")
            instance_options = discover.get_summary(ec2_client)
            enforcement = (
                float(instance_options["required"] / instance_options["instances"])
                * 100
            )
            if not json:
                instance_options["percent_enforcement_v2"] = utils.convert_green(
                    f"{enforcement:.2f}%"
                )
                return utils.pretty_metadata_summary([dict(instance_options)])
            elif json:
                instance_options["percent_enforcement_v2"] = enforcement
                print(utils.pretty_json_summary(instance_options))



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
import boto3
import logging
from metabadger.shared.utils import convert_red, convert_green, convert_yellow


def metamodify(
    ec2_client,
    action: str,
    httptoken: str,
    status: str,
    instance_id: str,
    dry_run: bool,
    region: str,
):
    """Helper function to change instance metadata status"""
    logging.basicConfig(
        filename="metabadger.log",
        format="%(asctime)s,%(message)s",
        level=logging.INFO,
        datefmt="%Y-%m-%dT%H:%M:%S",
    )
    if dry_run:
        status_color = convert_yellow("SUCCESS")
        status_text = "SUCCESS"
        print(
            f"IMDS updated (Dry run mode) : {action} for {instance_id:<80} {status_color:>20}"
        )
        logging.info(f"imds_updated_dry_run,{action},{instance_id},{status_text}")
    else:
        try:
            response = ec2_client.modify_instance_metadata_options(
                InstanceId=instance_id,
                HttpTokens=httptoken,
                HttpEndpoint=status,
            )
            status_color = convert_green("SUCCESS")
            status_text = "SUCCESS"
        except:
            status_color = convert_red("FAILED")
            status_text = "FAILED"
        print(
            f"IMDS updated : {action} in {region} for {instance_id:<80} {status_color:>20}"
        )
        logging.info(f"imds_updated,{action},{region},{instance_id},{status_text}")



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
""" Additional utils """
import csv
import json
import pandas as pd
from colorama import Fore, Back
from tabulate import tabulate

OK_GREEN = "\033[92m"
GREY = "\33[90m"
END = "\033[0m"


def print_yellow(string):
    """Print yellow text"""
    print(f"{Fore.YELLOW}{string}{END}")


def print_green(string):
    """Print green text"""
    print(f"{Fore.GREEN}{string}{END}")


def print_grey(string):
    """Print grey text"""
    print(f"{Fore.GREY}{string}{END}")


def print_red(string):
    """Print red text"""
    print(f"{Fore.RED}{string}{END}")


def convert_red(string):
    """Return red text"""
    return f"{Fore.RED}{string}{END}"


def convert_green(string):
    """Return green text"""
    return f"{Fore.GREEN}{string}{END}"


def convert_yellow(string):
    """Return green text"""
    return f"{Fore.YELLOW}{string}{END}"


def pretty_grid_keys(output: dict):
    """Print grid keys from dictionary"""
    df = pd.DataFrame(output)
    print(tabulate(df.T, headers="keys", tablefmt="grid"))


def read_from_csv(file):
    """Read file from a path and covert the csv into a list"""
    with open(file) as f:
        reader = csv.reader(f)
        data = list(reader)
    return data[0]


def pretty_metadata_summary(instance_options):
    """Format metadata usage breakdown"""
    print(tabulate(instance_options, headers="keys", tablefmt="grid"))


def pretty_json_summary(instance_dict):
    """Format metadata usage summary into JSON"""
    print(json.dumps(instance_dict, indent=4, sort_keys=True))



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
"""Disable IMDS service on instances"""
import click
from tabulate import tabulate
from metabadger.shared import utils, aws_auth, validate, discover, modify


@click.command(short_help="Disable the IMDS service on EC2 instances")
@click.option(
    "--exclusion",
    "-e",
    is_flag=True,
    default=False,
    help="The exclusion flag will apply to everything besides what is specified, tags or instances",
)
@click.option(
    "--dry-run",
    "-d",
    is_flag=True,
    default=False,
    help="Dry run of disabling the metadata service",
)
@click.option(
    "--input-file",
    "-i",
    type=click.Path(exists=True),
    required=False,
    help="Path of csv file of instances to disable IMDS for",
)
@click.option(
    "--tags",
    "-t",
    "tags",
    type=str,
    default="",
    help="A comma seperated list of tags to apply the hardening setting to",
    callback=validate.click_validate_tag_alphanumeric,
)
@click.option(
    "--region",
    "-r",
    "region",
    type=str,
    required=False,
    default="us-west-2",
    help="Specify which AWS region you will perform this command in",
)
@click.option(
    "--profile", "-p", type=str, required=False, help="Specify the AWS IAM profile."
)
def disable_metadata(
    dry_run: bool,
    input_file: str,
    tags: str,
    profile: str,
    region: str,
    exclusion: bool,
):
    ec2_resource = aws_auth.get_boto3_resource(
        region=region, profile=profile, service="ec2"
    )
    ec2_client = aws_auth.get_boto3_client(
        region=region, profile=profile, service="ec2"
    )
    instance_list = discover.discover_instances(ec2_client)
    if discover.discover_roles(ec2_client)[1]["role_count"] > 0:
        click.confirm(
            utils.convert_red(
                f"Warning: One or more of the instances in {ec2_client.meta.region_name} you want to update has a role attached, do you want to continue?"
            ),
            abort=True,
        )
    if discover.discover_roles(ec2_client)[1]["instance_count"] <= 0:
        utils.print_yellow(f"No EC2 instances found in region: {region}")
    if dry_run:
        utils.print_yellow(
            "Running in dry run mode, this will NOT make any changes to your metadata service"
        )
    if exclusion and input_file:
        utils.print_yellow("Excluding instances specified in your configuration file")
        data = utils.read_from_csv(input_file)
        print(f"Reading instances from input csv file\n{data}")
        delta = [instance for instance in instance_list if instance not in data]
        for instance in delta:
            modify.metamodify(
                ec2_client, "disabled", "optional", "disabled", instance, dry_run
            )
    elif exclusion and tags:
        utils.print_yellow("Excluding instances specified by tags")
        print(f"Tags: {tags}")
        for instance in instance_list:
            if not any(
                value in discover.get_instance_tags(ec2_client, instance)
                for value in tags
            ):
                modify.metamodify(
                    ec2_client, "disabled", "optional", "disabled", instance, dry_run
                )
    elif exclusion:
        utils.print_yellow("An exclusion requires either tags or instance list")
        raise click.Abort()
    elif input_file:
        data = utils.read_from_csv(input_file)
        print(f"Reading instances from input csv file\n{data}")
        for instance in data:
            modify.metamodify(
                ec2_client, "disabled", "optional", "disabled", instance, dry_run
            )
    elif tags:
        tag_apply_count = 0
        utils.print_yellow("Only applying hardening to the following tagged instances")
        print(f"Tags: {tags}")
        for instance in instance_list:
            if any(
                value in discover.get_instance_tags(ec2_client, instance)
                for value in tags
            ):
                modify.metamodify(
                    ec2_client, "disabled", "optional", "disabled", instance, dry_run
                )
                tag_apply_count += 1
        if tag_apply_count < 1:
            print(f"No instances with this tag found, no changes were made")
    else:
        for instance in instance_list:
            modify.metamodify(
                ec2_client, "disabled", "optional", "disabled", instance, dry_run
            )



# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
"""Discover IAM role usage which may conflict with altering IMDS"""
import click
from metabadger.shared import utils, aws_auth, discover


@click.option(
    "--region",
    "-r",
    "region",
    type=str,
    required=False,
    default="us-west-2",
    help="Specify which AWS region you will perform this command in",
)
@click.option(
    "--profile", "-p", type=str, required=False, help="Specify the AWS IAM profile."
)
@click.command(short_help="Discover summary of IAM role usage for EC2")
def discover_role_usage(profile: str, region: str):
    ec2_client = aws_auth.get_boto3_client(
        region=region, profile=profile, service="ec2"
    )
    instance_role_summary = discover.discover_roles(ec2_client)[0]
    if not instance_role_summary:
        utils.print_yellow(f"No attached EC2 IAM roles found in region: {region}")
    else:
        return utils.pretty_grid_keys(instance_role_summary)



#! /usr/bin/env python
# Copyright (c) 2021, salesforce.com, inc.
# All rights reserved.
# Licensed under the BSD 3-Clause license.
# For full license text, see the LICENSE file in the repo root
# or https://opensource.org/licenses/BSD-3-Clause
"""
    Metabadger is an AWS Security Tool used for discovering and hardening the Instance Metadata service.
"""
import click
from metabadger import command
from metabadger.bin.version import __version__


@click.group()
@click.version_option(version=__version__)
def metabadger():
    """
    Metabadger is an AWS Security Tool used for discovering and hardening the Instance Metadata service.
    """


metabadger.add_command(command.disable_metadata.disable_metadata)
metabadger.add_command(command.discover_metadata.discover_metadata)
metabadger.add_command(command.discover_role_usage.discover_role_usage)
metabadger.add_command(command.harden_metadata.harden_metadata)


def main():
    """ Metabadger is an AWS Security Tool used for discovering and hardening the Instance Metadata service. """
    metabadger()


if __name__ == "__main__":
    metabadger()
